from pyvdp.visadirect.models import VisaDirectTransactionModel, VisaDirectTransactionBatchModel


class PullFundsTransactionsModel(VisaDirectTransactionModel):
    """VisaDirect FundsTransfer PullFunds data object model.

    :param int systemsTraceAuditNumber: **Required**. Systems trace audit number.  6 digits integer.
    :param int acquiringBin: **Required**. Bank identification number (BIN) for VisaDirect acquiring. 6-11 digits
        integer.
    :param int acquirerCountryCode: **Required**. 
        `ISO country code <https://developer.visa.com/request_response_codes#isoCodes>`_ for acquiring institution.
    :param float amount: **Required**. Transaction amount nnn.mmm.
    :param str senderPrimaryAccountNumber: **Required**. Sender PAN (Primary Account Number), 13-19 characters.
    :param str senderCardExpiryDate: **Required**. Sender card expiration date, *YYYY-MM*, e.g. 2018-02.
    :param str senderCurrencyCode: **Required**. Sender currency code, 3 characters 
        `ISO currency code <https://developer.visa.com/request_response_codes#isoCodes>`_.
    :param str businessApplicationId: **Required**. 
        `Program business application type <https://developer.visa.com/request_response_codes#businessApplicationId>`_.
        2 characters string.
    :param CardAcceptorModel cardAcceptor: **Required**. Instance of :func:`~pyvdp.visadirect.CardAcceptorModel`.
    :param int merchantCategoryCode: **Conditional**. Merchant category code (MCC). If provided, overrides the one 
        present in onboarding data. If missing in onboarding, then this value is required. Max 4 digits integer.
    :param float surcharge: **Optional**. Sender surcharge, as assessed by originator. It must be the same currency and
        format as defined by **sender_currency_code**. Float 12 digits, max 3 fractions.
    :param string cavv: **Optional**. Cardholder Authentication Verification Value (CAVV), generated by ACS.
        Hexbin string, max 40 characters.
    :param str cardCvv2Value: **Optional**. Card CVV2 value for **sender_pan**. 3-4 characters string.
    :param float foreignExchangeFeeTransaction: Exchange fee on top of Visa exchange rate. Must be in the same
        currency and format as **amount**. Float 12 digits, max 3 fractions.
    :param MagneticStripeData magneticStripeData: **Optional**. 
        Instance of :func:`~pyvdp.visadirect.MagneticStripeData`.
    :param PointOfServiceDataModel pointOfServiceData: **Conditional**. 
        Instance of :func:`~pyvdp.visadirect.PointOfServiceDataModel`.
        Required for CardPresent transactions.
    :param PointOfServiceCapability: pointOfServiceCapability: **Conditional**. 
        Instance of :func:`~pyvdp.visadirect.PointOfServiceCapabilityModel`.
        Required for CardPresent transactions.
    :param PinData pinData: **Optional**. Instance of :func:`~pyvdp.visadirect.PinData`.
    :param str feeProgramIndicator: **Optional**. You probably don't need it.
    :param str merchantPseudoAbaNumber: **Optional**. Unique ID for originator when transaction is sent via PPGS.
        9 characters string.
    :param str sharingGroupCode: **Optional**. Used by PPG participants to specify network access priority.
        See `Sharing Group Code <https://developer.visa.com/request_response_codes#networkIDAndSharingCode>`_
    :param MerchantVerificationValueModel merchantVerificationValue: **Optional**. 
        Instance of :func:`~pyvdp.visadirect.MerchantVerificationValueModel`.
    :param bool multi: **Conditional**. If True, this transaction is a part of batch. 
        See :func:`~pyvdp.visadirect.fundstransfer.MultiPullFundsTransactionsModel`

    **Request:**
    
    ..  code:: json

        {
            "acquirerCountryCode": 840,
            "acquiringBin": 408999,
            "amount": 124.02,
            "businessApplicationId": "AA",
            "cardAcceptor": {
                "address": {
                    "country": "USA",
                    "county": "San Mateo",
                    "state": "CA",
                    "zipCode": "94404"
                },
                "idCode": "ABCD1234ABCD123",
                "name": "Visa Inc. USA-Foster City",
                "terminalId": "ABCD1234"
            },
            "cavv": "0700100038238906000013405823891061668252",
            "foreignExchangeFeeTransaction": "11.99",
            "localTransactionDateTime": "2017-03-17T08:20:42",
            "retrievalReferenceNumber": "330000550000",
            "senderCardExpiryDate": "2015-10",
            "senderCurrencyCode": "USD",
            "senderPrimaryAccountNumber": "4895142232120006",
            "surcharge": "11.99",
            "systemsTraceAuditNumber": "451001"
        }

    **Response:**
    
    ..  code:: json
    
        {
            "transactionIdentifier": 587010322176104,
            "actionCode": "00",
            "approvalCode": "98765X",
            "responseCode": "5",
            "transmissionDateTime": "2017-04-20T04:41:12.000Z",
            "cavvResultCode": "8"
        }    
    """
    ATTRS = [
        'acquiringBin',
        'acquirerCountryCode',
        'amount',
        'senderPrimaryAccountNumber',
        'senderCardExpiryDate',
        'senderCurrencyCode',
        'businessApplicationId',
        'cardAcceptor',
        'cavv',
        'surcharge',
        'cardCvv2Value',
        'foreignExchangeFeeTransaction',
        'merchantCategoryCode',
        'magneticStripeData',
        'pointOfServiceData',
        'pointOfServiceCapability',
        'pinData',
        'feeProgramIndicator',
        'merchantPseudoAbaNumber',
        'sharingGroupCode',
        'merchantVerificationValue'
    ]

    def __init__(self, **kwargs):
        super(PullFundsTransactionsModel, self).__init__(**kwargs)
        for attr, value in kwargs.items():
            if attr in self.ATTRS and value:
                self.__setattr__(attr, value)


class MultiPullFundsTransactionsModel(VisaDirectTransactionBatchModel):
    """VisaDirect FundsTransfer MultiPullFunds data object model.

    It is container object, which incorporate PullFundsTransactionsModel objects under the same acquirer
    credentials.
    
    ..  note::
        
        When instance of PullFundsTransactionsModel incorporated in MultiPullFundsTransactionsModel, 
        attributes `acquirerBin`, `acquiringCountryCode` and `businessApplicationId` are migrating from 
        PullFundsTransactionsModel to  MultiPullFundsTransactionsModel level.    

    :param int acquiringBin: **Required**. Bank identification number (BIN) for VisaDirect acquiring. 6-11 digits
        integer.
    :param int acquirerCountryCode: **Required**. 
        `ISO country code <https://developer.visa.com/request_response_codes#isoCodes>`_ for acquiring institution.
    :param str businessApplicationId: **Required**. `Program business application type 
        <https://developer.visa.com/request_response_codes#businessApplicationId>`_. 2 characters string.
    :param list request: **Required**. List of :func:`~pyvdp.visadirect.fundstransfer.PullFundsTransactionsModel`
        objects. 

    **Request:**
    
    ..  code:: json

        {
            "acquirerCountryCode": "608",
            "acquiringBin": "408999",
            "businessApplicationId": "AA",
            "localTransactionDateTime": "2017-03-17T09:11:08",
            "merchantCategoryCode": "6012",
            "request": [
                {
                    "amount": "100.00",
                    "cardAcceptor": {
                        "address": {
                            "country": "USA",
                            "county": "00",
                            "state": "CA",
                                "zipCode": "94454"
                        },
                        "idCode": "5678",
                        "name": "Mr Smith",
                        "terminalId": "1234"
                    },
                    "cavv": "0700020718799100000002980179911000000000",
                    "localTransactionDateTime": "2017-03-17T09:11:08",
                    "retrievalReferenceNumber": "401010101011",
                    "senderCardExpiryDate": "2020-12",
                    "senderCurrencyCode": "USD",
                    "senderPrimaryAccountNumber": "4895140000066666",
                    "systemsTraceAuditNumber": "101011"
                },
                {
                    "amount": "100.00",
                    "cardAcceptor": {
                        "address": {
                            "country": "USA",
                            "county": "00",
                            "state": "CA",
                            "zipCode": "94454"
                        },
                        "idCode": "5678",
                        "name": "Mr Smith",
                        "terminalId": "1234"
                    },
                    "cavv": "0700020718799100000002980179911000000000",
                    "localTransactionDateTime": "2017-03-17T09:11:08",
                    "retrievalReferenceNumber": "401010101011",
                    "senderCardExpiryDate": "2020-12",
                    "senderCurrencyCode": "USD",
                    "senderPrimaryAccountNumber": "4895140000066666",
                    "systemsTraceAuditNumber": "101011"
                }
            ]
        }
        
    **Response:**
    
    ..  code:: text
    
        "1492665372_866_68_l73c002_VDP_ARM"
    
    """
    ATTRS = [
        'acquiringBin',
        'acquirerCountryCode',
        'businessApplicationId',
        'request'
    ]

    def __init__(self, **kwargs):
        super(MultiPullFundsTransactionsModel, self).__init__(request=kwargs['request'])
        for attr, value in kwargs.items():
            if attr in self.ATTRS and value:
                self.__setattr__(attr, value)


class PushFundsTransactionsModel(VisaDirectTransactionModel):
    """VisaDirect FundsTransfer PushFundsTransactions data object model.

    :param int systemsTraceAuditNumber: **Required**. Systems trace audit number.  6 digits integer.
    :param int acquiringBin: **Required**. Bank identification number (BIN) for VisaDirect acquiring.
        6-11 digits integer.
    :param int acquirerCountryCode: **Required**. 
        `ISO country code <https://developer.visa.com/request_response_codes#isoCodes>`_ for acquiring institution.
    :param str transactionCurrencyCode: **Required**. 
        `ISO currency code <https://developer.visa.com/request_response_codes#isoCodes>`_.
        Recommended to use value from :func:`~pyvdp.paai.fundstransferattinq.cardattributes.fundstransferinquiry.send`
        unless prohibited by local law. 3 characters string.
    :param str recipientPrimaryAccountNumber: **Required**. Recipient primary account number (PAN). 
        13-19 characters string.
    :param float amount: **Required**. Transaction amount. 12 digits, 3 fractions.
    :param businessApplicationId: **Required**. 
        `Business application type <https://developer.visa.com/request_response_codes#businessApplicationId>`_ for 
        VisaNet processing. 2 characters string.
    :param CardAcceptorModel cardAcceptor: **Required**. Instance of :func:`~pyvdp.visadirect.CardAcceptorModel`.
    :param str senderReference: **Conditional**. Required, if transaction is money transfer, pre-paid load, credit card
        bill pay and if sender is funding with a non financial instrument (cash). Required if funds disbursement.
        Required if **senderAccountNumber** is not set. Max 16 characters string.
    :param str senderAccountNumber: **Conditional**.  If the transaction is a money transfer, pre-paid load, or
        credit card bill pay, and if the sender intends to fund the transaction with a financial instrument (for
        example, debit card), this field is required and must contain the sender's account number. If the transaction
        is a funds disbursement, the field is not required. Max 34 characters string.
    :param str senderAddress: **Conditional**. Sender address. Max 35 characters string.
    :param str senderCountryCode: **Conditional**. 
        Sender `ISO country code <https://developer.visa.com/request_response_codes#isoCodes>`_.
        Required for cross-border and US domestic money transfers. 2-3 characters string.
    :param str senderName: **Conditional**. Required for cross-border and US domestic money transfers. Max 30
        characters string.
    :param str senderCity: **Conditional**. Required for cross-border and US domestic money transfers. Max 25
        characters string.
    :param str senderStateCode: **Conditional**. Required if **senderCountryCode** is 124(CAN) or 840(USA). 2 characters
        string.
    :param str recipientName: **Conditional**. Required for cross-border. Max 30 characters string.
    :param int merchantCategoryCode: **Conditional**. Merchant category code. Overrides the one provided with 
        onboarding data. Required if not provided during onboarding. 4 digits integer.
    :param int transactionIdentifier: **Conditional**. VisaNet transaction ID. Provide if OCT preceeded by AFT.
        Should be populated with value, returned with AFT. 15 digits integer.
    :param str sourceOfFundsCode: **Conditional**. 
        Refer to `sourceOfFundsCode <https://developer.visa.com/request_response_codes#sourceOfFundsCode>`_ codes
    :param str recipientCardExpiryDate: **Optional**. Recipient card expiration for **recipient_pan**, YYYY-MM.
    :param str accountType: **Optional**. Used to define account type of the sender PAN. Default '00' (Not applicable).
    :param str feeProgramIndicator: **Optional**.
    :param str senderDateOfBirth: **Optional**. Sender's date of birth, YYYY-MM-DD.
    :param PointOfServiceDataModel pointOfServiceData: **Conditional**. 
        Instance of :func:`~pyvdp.visadirect.PointOfServiceDataModel`. Required for CardPresent transactions.
    :param float surcharge: **Optional**. Sender's surcharge as assessed by originator. Must be in the same format
        and currency as **amount**. 12 digits, 3 fractions float.
    :param str merchantPseudoAbaNumber: **Optional**. Uniquely identifies originator with PPGS. 9 characters string.
    :param str sharingGroupCode: **Optional**. Used by PPG participants to specify network priority. 30 characters
        string.
    :param MerchantVerificationValueModel merchantVerificationValue: **Optional**. 
        Instance of :func:`~pyvdp.visadirect.MerchantVerificationValueModel`.

    **Request:**
    
    ..  code:: json

        {
            "acquirerCountryCode": "840",
            "acquiringBin": "408999",
            "amount": "124.05",
            "businessApplicationId": "AA",
            "cardAcceptor": {
                "address": {
                    "country": "USA",
                    "county": "San Mateo",
                    "state": "CA",
                    "zipCode": "94404"
                },
                "idCode": "CA-IDCode-77765",
                "name": "Visa Inc. USA-Foster City",
                "terminalId": "TID-9999"
            },
            "localTransactionDateTime": "2017-03-17T09:01:14",
            "merchantCategoryCode": "6012",
            "pointOfServiceData": {
                "motoECIIndicator": "0",
                "panEntryMode": "90",
                "posConditionCode": "00"
            },
            "recipientName": "rohan",
            "recipientPrimaryAccountNumber": "4957030420210462",
            "retrievalReferenceNumber": "412770451018",
            "senderAccountNumber": "4957030420210454",
            "senderAddress": "901 Metro Center Blvd",
            "senderCity": "Foster City",
            "senderCountryCode": "124",
            "senderName": "Mohammed Qasim",
            "senderReference": "",
            "senderStateCode": "CA",
            "sourceOfFundsCode": "05",
            "systemsTraceAuditNumber": "451018",
            "transactionCurrencyCode": "USD",
            "transactionIdentifier": "381228649430015"
        }
        
    **Response:**
    
    ..  code:: json
        
        {
            "transactionIdentifier": 587010322176105,
            "actionCode": "00",
            "approvalCode": "21324K",
            "responseCode": "5",
            "transmissionDateTime": "2017-04-21T03:47:49.000Z"
        }        
    """
    ATTRS = [
        'amount',
        'acquiringBin',
        'acquirerCountryCode',
        'transactionCurrencyCode',
        'recipientPrimaryAccountNumber',
        'businessApplicationId',
        'cardAcceptor',
        'senderReference',
        'senderAccountNumber',
        'senderAddress',
        'senderCountryCode',
        'senderName',
        'senderCity',
        'senderStateCode',
        'recipientName',
        'merchantCategoryCode',
        'transactionIdentifier',
        'sourceOfFundsCode',
        'recipientCardExpiryDate',
        'accountType',
        'feeProgramIndicator',
        'senderDateOfBirth',
        'pointOfServiceData',
        'surcharge',
        'merchantPseudoAbaNumber',
        'sharingGroupCode',
        'merchantVerificationValue'
    ]

    def __init__(self, **kwargs):
        super(PushFundsTransactionsModel, self).__init__(**kwargs)
        for attr, value in kwargs.items():
            if attr in self.ATTRS and value:
                self.__setattr__(attr, value)


class MultiPushFundsTransactionsModel(VisaDirectTransactionBatchModel):
    """VisaDirect FundsTransfer MultiPushFundsTransactions data object model.

    It is container object, which incorporate PushFundsTransactionsModel objects under the same acquirer
    credentials. 
    
    ..  note::
        
        When instance of PushFundsTransactionsModel incorporated in MultiPushFundsTransactionsModel, attributes 
        `acquirerBin`, `acquiringCountryCode` and `businessApplicationId` are migrating from 
        PushFundsTransactionsModel to MultiPushFundsTransactionsModel level.

    :param int acquiringBin: **Required**. Bank identification number (BIN) for VisaDirect acquiring. 6-11 digits
        integer.
    :param int acquirerCountryCode: **Required**. 
        `ISO country code <https://developer.visa.com/request_response_codes#isoCodes>`_ for acquiring institution.
    :param str businessApplicationId: **Required**. 
        `Program business application type <https://developer.visa.com/request_response_codes#businessApplicationId>`_.
        2 characters string.
    :param list request: **Required**. List of :func:`~pyvdp.visadirect.fundstransfer.PushFundsTransactionsModel`
        instances. 

    **Request:**
    
    ..  code:: json

        {
            "acquirerCountryCode": "840",
            "acquiringBin": "408999",
            "businessApplicationId": "AA",
            "localTransactionDateTime": "2017-03-17T09:12:46",
            "merchantCategoryCode": "6012",
            "request": [
                {
                    "amount": "100.00",
                    "cardAcceptor": {
                        "address": {
                            "country": "USA",
                            "county": "00",
                            "state": "CA",
                            "zipCode": "94454"
                        },
                        "idCode": "5678",
                        "name": "Mr Smith",
                        "terminalId": "1234"
                    },
                    "feeProgramIndicator": "123",
                    "localTransactionDateTime": "2017-03-17T09:12:46",
                    "recipientName": "Akhila",
                    "recipientPrimaryAccountNumber": "4957030420210454",
                    "retrievalReferenceNumber": "401010101011",
                    "senderAccountNumber": "4005520000011126",
                    "senderAddress": "My Address",
                    "senderCity": "My City",
                    "senderCountryCode": "USA",
                    "senderName": "Mr Name",
                    "senderReference": "",
                    "senderStateCode": "CA",
                    "sourceOfFundsCode": "01",
                    "systemsTraceAuditNumber": "101011",
                    "transactionCurrencyCode": "USD",
                    "transactionIdentifier": "234234234234234"
                },
                { ... }
            ]
        }
        
    **Response:**
    
    ..  code:: text
    
        "1492746769_916_89_l73c001_VDP_ARM"
    
    """
    ATTRS = [
        'acquiringBin',
        'acquirerCountryCode',
        'businessApplicationId',
        'request'
    ]

    def __init__(self, **kwargs):
        super(MultiPushFundsTransactionsModel, self).__init__(request=kwargs['request'])
        for attr, value in kwargs.items():
            if attr in self.ATTRS and value:
                self.__setattr__(attr, value)


class ReverseFundsTransactionsModel(PullFundsTransactionsModel):
    """VisaDirect FundsTransfer ReverseFundsTransactions data object model.

    :param int systemsTraceAuditNumber: **Required**. Systems trace audit number.  6 digits integer.
    :param int transactionIdentifier: **Required**. VisaNet transaction ID. This field must be populated with
        `transactionIdentifier` from initial AFT.
    :param int acquiringBin: **Required**. Bank identification number (BIN) for VisaDirect acquiring.
        6-11 digits integer.
    :param int acquirerCountryCode: **Required**. 
        `ISO country code <https://developer.visa.com/request_response_codes#isoCodes>`_ for acquiring institution.
    :param str senderPrimaryAccountNumber: **Required**. Primary account number of the sender. 13-19 characters string.
    :param str senderCardExpiryDate: **Optional**. Sender card expiration date, YYYY-MM.
    :param str senderCurrencyCode: **Required**. 
        `ISO currency code <https://developer.visa.com/request_response_codes#isoCodes>`_. 3 characters string.
    :param float amount: **Required**. Transaction amount. 12 digits, 3 fractions.
    :param float surcharge: **Optional**. Sender's surcharge as assessed by originator. Must be in the same format
        and currency as **amount**. 12 digits, 3 fractions float.
    :param float foreignExchangeFeeTransaction: **Optional**. Foreign exchange markup fee, on top of VisaNet
        exchange rate. 12 digist, 3 fractions float.
    :param OriginalDataElementsModel originalDataElements: **Required**. 
        Instance of :func:`~pyvdp.visadirect.OriginalDataElementsModel`.
    :param CardAcceptorModel cardAcceptor: **Required**. Instance of :func:`~pyvdp.visadirect.CardAcceptorModel`.
    :param PointOfServiceDataModel pointOfServiceData: **Optional**. 
        Instance of :func:`~pyvdp.visadirect.PointOfServiceDataModel`.
    :param PointOfServiceCapabilityModel pointOfServiceCapability: **Optional**. 
        Instance of :func:`~pyvdp.visadirect.PointOfServiceCapabilityModel`.
    :param str feeProgramIndicator: **Optional**.
    :param str merchantPseudoAbaNumber: **Optional**. Uniquely identifies originator with PPGS. 9 characters string.
    :param str sharingGroupCode: **Optional**. Used by PPG participants to specify network priority. 30 characters
        string.
    :param int networkId: **Optional**. Code, that specifies network for transmission. 
        Refer to `Network ID <https://developer.visa.com/request_response_codes#networkIDAndSharingCode>`_. 2 digits 
        integer.
    :param MerchantVerificationValueModel merchantVerificationValue: **Optional**. 
        Instance of :func:`~pyvdp.visadirect.MerchantVerificationValueModel`.

    **Request:**
    
    ..  code:: json
        
        {
            "acquirerCountryCode": "608",
            "acquiringBin": "408999",
            "amount": "24.01",
            "cardAcceptor": {
                "address": {
                    "country": "USA",
                    "county": "San Mateo",
                    "state": "CA",
                    "zipCode": "94404"
                },
                "idCode": "VMT200911026070",
                "name": "Visa Inc. USA-Foster City",
                "terminalId": "365539"
            },
            "localTransactionDateTime": "2017-04-21T03:53:39",
            "originalDataElements": {
                "acquiringBin": "408999",
                "approvalCode": "20304B",
                "systemsTraceAuditNumber": "897825",
                "transmissionDateTime": "2017-04-21T03:53:39"
            },
            "pointOfServiceCapability": {
                "posTerminalEntryCapability": "2",
                "posTerminalType": "4"
            },
            "pointOfServiceData": {
                "motoECIIndicator": "0",
                "panEntryMode": "90",
                "posConditionCode": "00"
            },
            "retrievalReferenceNumber": "330000550000",
            "senderCardExpiryDate": "2015-10",
            "senderCurrencyCode": "USD",
            "senderPrimaryAccountNumber": "4895100000055127",
            "systemsTraceAuditNumber": "451050",
            "transactionIdentifier": "381228649430011"
        }        

    **Response:**
    
    ..  code:: json
    
        {
            "transactionIdentifier": 587010322176103,
            "approvalCode": "23456M",
            "actionCode": "00",
            "responseCode": "5",
            "transmissionDateTime": "2017-04-21T03:54:36.000Z"
        }    
    """
    ATTRS = [
        'transactionIdentifier',
        'acquiringBin',
        'acquirerCountryCode',
        'senderPrimaryAccountNumber',
        'senderCardExpiryDate',
        'senderCurrencyCode',
        'amount',
        'surcharge',
        'foreignExchangeFeeTransaction',
        'originalDataElements',
        'cardAcceptor',
        'pointOfServiceData',
        'pointOfServiceCapability',
        'feeProgramIndicator',
        'merchantPseudoAbaNumber',
        'sharingGroupCode',
        'networkId',
        'merchantVerificationValue'
    ]

    def __init__(self, **kwargs):
        super(ReverseFundsTransactionsModel, self).__init__(**kwargs)
        for attr, value in kwargs.items():
            if attr in self.ATTRS and value:
                self.__setattr__(attr, value)


class MultiReverseFundsTransactionsModel(MultiPullFundsTransactionsModel):
    """VisaDirect FundsTransfer MultiReverseFundsTransactions data object model.

    It is container object, which incorporate ReverseFundsTransactionsModel objects under the same acquirer credentials.

    :param int acquiringBin: **Required**. Bank identification number (BIN) for VisaDirect acquiring. 6-11 digits
        integer.
    :param int acquirerCountryCode: **Required**. 
        `ISO country code <https://developer.visa.com/request_response_codes#isoCodes>`_ for acquiring institution.
    :param list request: **Required**. List of :func:`~pyvdp.visadirect.fundstransfer.ReverseFundsTransactionsModel`
        instances. 
        
    **Request:**
    
    ..  code:: json

        {
            "acquirerCountryCode": "840",
            "acquiringBin": "408999",
            "localTransactionDateTime": "2017-04-21T03:56:17",
            "request": [
                {
                    "amount": "100.00",
                    "cardAcceptor": {
                        "address": {
                            "country": "USA",
                            "county": "00",
                            "state": "CA",
                            "zipCode": "94454"
                        },
                        "idCode": "5678",
                        "name": "Mr Smith",
                        "terminalId": "1234"
                    },
                    "localTransactionDateTime": "2017-04-21T03:56:17",
                    "originalDataElements": {
                        "acquiringBin": "408999",
                        "approvalCode": "1ABCDE",
                        "systemsTraceAuditNumber": "228112",
                        "transmissionDateTime": "2017-04-21T03:56:17"
                    },
                    "retrievalReferenceNumber": "401010101011",
                    "senderCardExpiryDate": "2020-12",
                    "senderCurrencyCode": "USD",
                    "senderPrimaryAccountNumber": "4485810000000131",
                    "systemsTraceAuditNumber": "101011",
                    "transactionIdentifier": "101010101010"
                },
                { ... }
            ]
        }
        
    **Response:**
    
    ..  code:: text
    
        "1492747084_010_78_l73c037_VDP_ARM"
    
    """
    ATTRS = [
        'acquiringBin',
        'acquirerCountryCode',
        'request'
    ]

    def __init__(self, **kwargs):
        super(MultiReverseFundsTransactionsModel, self).__init__(request=kwargs['request'])
        for attr, value in kwargs.items():
            if attr in self.ATTRS and value:
                self.__setattr__(attr, value)
